@@ -223,7 +223,7 @@ class Client:
                         '     * @object: A #${camelcase}.\n'
                         '     * @output: A #${output_camelcase}.\n'
                         '     *\n'
-                        '     * The ::${signal_name} signal gets emitted when a \'<link linkend=\"libqmi-glib-${service}-${message_name_dashed}.top_of_page\">${message_name}</link>\' indication is received.\n'
+                        '     * The ::${signal_name} signal gets emitted when a \'<link linkend=\"libqmi-glib-${service}-${message_name_dashed}-indication.top_of_page\">${message_name}</link>\' indication is received.\n'
                         '     *\n'
                         '     * Since: ${since}\n'
                         '     */\n'
@@ -487,6 +487,12 @@ class Client:
                 '                                        callback,\n'
                 '                                        user_data,\n'
                 '                                        ${underscore}_${message_underscore});\n'
+                '    if (!__qmi_client_is_valid (QMI_CLIENT (self))) {\n'
+                '        g_simple_async_result_set_error (result, QMI_CORE_ERROR, QMI_CORE_ERROR_WRONG_STATE, "client invalid");\n'
+                '        g_simple_async_result_complete_in_idle (result);\n'
+                '        g_object_unref (result);\n'
+                '        return;\n'
+                '    }\n'
                 '\n'
                 '    transaction_id = qmi_client_get_next_transaction_id (QMI_CLIENT (self));\n'
                 '\n'
