@@ -740,11 +740,13 @@ __qmi_utils_get_driver (const gchar *cdc_wdm_path)
          *    /sys/bus/usb/drivers/qmi_wwan
          */
         tmp = g_strdup_printf ("/sys/class/%s/%s/device/driver", subsystems[i], device_basename);
-        path = canonicalize_file_name (tmp);
+        path = realpath (tmp, NULL);
         g_free (tmp);
 
-        if (g_file_test (path, G_FILE_TEST_EXISTS))
-            driver = g_path_get_basename (path);
+        if (!path)
+            continue;
+
+        driver = g_path_get_basename (path);
         g_free (path);
     }
 
